package com.csg.cs.ws.locator;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;

import javax.xml.ws.BindingProvider;

import com.csg.cs.corex.ctxtransmitter.ws.consumer.WsHeaderPropagator;

public class SEIContextDecoratorFactory {

  private static class WsHeaderPropagatorBasedSEIContext implements SEIContext {
    private BindingProvider provider = null;

    WsHeaderPropagatorBasedSEIContext(BindingProvider provider) {
      this.provider = provider;
    }

    public void setInterface(String interf) {
      WsHeaderPropagator.setInterface(interf, provider);
    }

    public void setInterfaceGroup(String interfaceGroup) {
      WsHeaderPropagator.setInterfaceGroup(interfaceGroup, provider);
    }

    public void setTargetBUId(String targetBUId) {
      WsHeaderPropagator.setTargetBUId(targetBUId, provider);
    }
  }

  public static Object createWsHeaderPropagatorEnabledDecorator(
      BindingProvider provider) {
    // Register the WS-handler to propagate WsHeader values to service
    // implementation.
    WsHeaderPropagator.registerTo(provider);
    WsHeaderPropagatorBasedSEIContext ctx = new WsHeaderPropagatorBasedSEIContext(
        provider);
    return createDecorator(ctx, provider);
  }

  public static Object createDecorator(SEIContext context,
      Object objectToDecorate) {
    final SEIContext ctx = context;
    final Object objToDecorate = objectToDecorate;
    InvocationHandler handler = new InvocationHandler() {
      public Object invoke(Object proxy, Method method, Object[] args)
          throws Throwable {
        try {
          if (method.getDeclaringClass().equals(SEIContext.class)) {
            return method.invoke(ctx, args);
          } else {
            return method.invoke(objToDecorate, args);
          }
        } 
        catch (InvocationTargetException ie) {
          Throwable cause = ie.getCause();
          throw cause == null ? ie : cause;
        }
      }
    };
    Class<?>[] baseInterfaces = objToDecorate.getClass().getInterfaces();
    Class<?>[] allInterfaces = new Class[baseInterfaces.length + 1];
    System
        .arraycopy(baseInterfaces, 0, allInterfaces, 0, baseInterfaces.length);
    allInterfaces[allInterfaces.length - 1] = SEIContext.class;
    return Proxy.newProxyInstance(SEIContextDecoratorFactory.class
        .getClassLoader(), allInterfaces, handler);
  }

}
