package com.csg.cs.ws.locator;

import java.net.URI;
import java.net.URISyntaxException;

public class SEIContextFactory {

  private SEIContextFactory() {
  }
  
  public static SEIContextImpl createSEIContext(String uri) {
    if (uri == null) {
      return createSEIContext(null, null, null);
    }
    try {
      URI u = new URI(uri);
      return createSEIContext(u);
    } catch (URISyntaxException e) {
      throw new IllegalArgumentException("malformed URI '" + uri + "'", e);
    }
  }
  
  public static SEIContextImpl createSEIContext(URI uri) {
    if (uri == null) {
      return createSEIContext(null, null, null);
    }
    if (!"csxb".equalsIgnoreCase(uri.getScheme())) {
      throw new IllegalArgumentException("Illegal uri '" + uri + "': expected scheme is csxb");
    }
    String targetBUId = uri.getAuthority();
    String interfaceGroup = null;
    String interf = null;
    String[] pElements = uri.getPath().split("/"); //first element is an empty string
    if (pElements.length >= 2) {
      interfaceGroup = pElements[1];
    }
    if (pElements.length >= 3) {
      interf = pElements[2];
    }
    return createSEIContext(targetBUId, interfaceGroup, interf);
  }

  public static SEIContextImpl createSEIContext(String targetBUId, String interfaceGroup, String interf) {
    return new SEIContextImpl(targetBUId, interfaceGroup, interf);
  }
  
}
