package com.csg.cs.ws.locator;

import java.util.Deque;
import java.util.LinkedList;

/**
 * An SEILocator that implements a stack. After usage of an SEI the method
 * {@link #pushSEI(Object)} has to be used to push back the SEI to the stack.
 * <p>
 * If client code is running in an EJB it's recommended to store SEI simply in
 * an EJB private member.
 * 
 * @author A530732
 * 
 * @param <SEI>
 *          Generated JAW-WS service end-point interface
 */
public final class StackedSEILocator<SEI> implements SEILocator<SEI> {

  private final Deque<SEI> dequeOfSEIs = new LinkedList<SEI>();

  /**
   * SEILocator to delegate to.
   */
  SEILocator<SEI> delegate = null;

  public StackedSEILocator(SEILocator<SEI> delegate) {
    this.delegate = delegate;
  }


  public SEI getSEI() {
    return popSEI();
  }

  public SEI popSEI() {
    SEI sei = null;
    synchronized (dequeOfSEIs) {
      if (!dequeOfSEIs.isEmpty()) {
        sei = dequeOfSEIs.pop();
      }
    }
    if (sei == null) {
      sei = delegate.getSEI();
    }
    return sei;
  }

  /**
   * Method used to push back a SEI to the stack after each usage.
   * 
   * @param sei Service interface
   */
  public void pushSEI(SEI sei) {
    synchronized (dequeOfSEIs) {
      dequeOfSEIs.push(sei);
    }
  }
}
