package com.csg.cs.ws.locator;


/**
 * ThreadLocal to manage generated JAX-WS service end-point interface (SEI) in a
 * thread safe way. According to JAX-WS specification the SEI is not thread
 * safe.
 * 
 * <p>
 * If client code is running in an EJB it's recommended to store SEI simply in
 * an EJB private member, not in a ThreadLocal.
 * 
 * @author A530732
 * 
 * @param <SEI>
 *          Generated JAW-WS service end-point interface
 */
public final class ThreadLocalSEILocator<SEI> extends ThreadLocal<SEI> implements
    SEILocator<SEI> {

  /**
   * SEILocator to delegate to.
   */
  SEILocator<SEI> delegate = null;

  public ThreadLocalSEILocator(SEILocator<SEI> delegate) {
    this.delegate = delegate;
  }

  /**
   * Encapsulates CS specific lookup of an SEI.
   */
  @SuppressWarnings( {"deprecation", "unchecked"})
  @Override
  protected SEI initialValue() {
    return delegate.getSEI();
  }

  
  public SEI getSEI() {
    return get();
  }

}
