package com.csg.cs.ws.locator.bridge;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashMap;
import java.util.Map;

import javax.xml.ws.WebServiceException;

import com.csg.cs.core.base.config.ConfigException;
import com.csg.cs.core.base.config.ConfigHelper;
import com.csg.cs.core.base.config.Configuration;
import com.csg.cs.security.wls.CSUtils;
import com.csg.cs.ws.locator.SEIContext;
import com.csg.cs.ws.locator.SEIContextDecoratorFactory;
import com.csg.cs.ws.locator.SEIContextFactory;
import com.csg.cs.ws.locator.SEIContextImpl;
import com.csg.cs.ws.locator.SEILocator;
import com.csg.cs.ws.locator.SmartSEILocatorFactory;
import com.csg.cs.ws.locator.j2ee.WlsSEILocator;
import com.csg.cs.ws.locator.jse.JseSEILocator;

/**
 * 
 * @author A530732
 * 
 * @param <SRV>
 *          Generated JAX-WS service interface
 * @param <SEI>
 *          Generated JAW-WS service end-point interface (port type)
 */
public final class SEILocatorBridge<SRV, SEI> implements SEILocator<SEI> {

  private Class<SEI> seiClass = null; // 

  private Map<LOCATOR_TYPE, SEILocator<SEI>> locators = new HashMap<LOCATOR_TYPE, SEILocator<SEI>>(
      2);

  private Map<String, LOCATOR_TYPE> targetBUIdRelatedLocatorTypes = new HashMap<String, LOCATOR_TYPE>();

  public static enum LOCATOR_TYPE {
    EJB, WS
  }

  private String uri = null;

    /**
     *
      * @param serviceClass service class
     * @param serviceEndpointInterfaceClass endpoint
     * @param portName port type name
     * @param address address (URI)
     */
  public SEILocatorBridge(Class<SRV> serviceClass,
      Class<SEI> serviceEndpointInterfaceClass, String portName, String address) {
    init(serviceClass, serviceEndpointInterfaceClass, portName, address);
  }

  @SuppressWarnings("unchecked")
  public SEILocatorBridge(Map<String, String> params) {
    Class<SRV> serviceClass = (Class<SRV>)getClassParam(params, "serviceClass");
    Class<SEI> serviceEndpointInterfaceClass = (Class<SEI>)getClassParam(
        params, "serviceEndpointInterfaceClass");
    String portName = getStringParam(params, "portName");
    String address = getStringParam(params, "wsseServiceName");
    uri = getStringParam(params, "uri");

    this.seiClass = serviceEndpointInterfaceClass;
    initTargetBUIdRelatedLocatorTypes(params, LOCATOR_TYPE.WS);
    initTargetBUIdRelatedLocatorTypes(params, LOCATOR_TYPE.EJB);
    if (hasTargetBUIdRelatedLocatorTye(LOCATOR_TYPE.WS)) {
      if (isInWlsServer()) {
        locators.put(LOCATOR_TYPE.WS, new WlsSEILocator<SRV, SEI>(serviceClass,
            serviceEndpointInterfaceClass, portName, address));
      } else {
        locators.put(LOCATOR_TYPE.WS, new JseSEILocator<SRV, SEI>(serviceClass,
            serviceEndpointInterfaceClass, getMappedAddress(LOCATOR_TYPE.WS,
                address)));
        // endpoint address is null. Use
        // ((JseSEILocator)getSEILocator(LOCATOR_TYPE.WS)).setEndpointAddress()
      }
    }
    if (hasTargetBUIdRelatedLocatorTye(LOCATOR_TYPE.EJB)) {
      locators
          .put(
              LOCATOR_TYPE.EJB,
              SmartSEILocatorFactory
                  .createLocatingPerSEIMethodInvocationSEILocator(new EjbSEILocator<SEI>(
                      serviceEndpointInterfaceClass, getStringParam(params,
                          "EJBAddress"))));
    }
  }

  private void init(Class<SRV> serviceClass,
      Class<SEI> serviceEndpointInterfaceClass, String portName, String address) {
    this.seiClass = serviceEndpointInterfaceClass;
    if (isInWlsServer()) {
      locators.put(LOCATOR_TYPE.WS, new WlsSEILocator<SRV, SEI>(serviceClass,
          serviceEndpointInterfaceClass, portName, address));
    } else {
      locators.put(LOCATOR_TYPE.WS, new JseSEILocator<SRV, SEI>(serviceClass,
          serviceEndpointInterfaceClass, getMappedAddress(LOCATOR_TYPE.WS,
              address)));
      // endpoint address is null. Use
      // ((JseSEILocator)getSEILocator(LOCATOR_TYPE.WS)).setEndpointAddress()
    }
    locators.put(LOCATOR_TYPE.EJB, SmartSEILocatorFactory
        .createLocatingPerSEIMethodInvocationSEILocator(new EjbSEILocator<SEI>(
            serviceEndpointInterfaceClass, getMappedAddress(LOCATOR_TYPE.EJB,
                address))));
    initTargetBUIdRelatedLocatorTypes(serviceEndpointInterfaceClass);
  }

  private String getStringParam(Map<String, String> params, String param) {
    String p = params.get(param);
    if (p == null) {
      throw new WebServiceException("mandatory parameter " + param + " missing");
    }
    return p;
  }

  private Class<?> getClassParam(Map<String, String> params, String param) {
    String p = getStringParam(params, param);
    try {
      return Class.forName(p);
    } catch (ClassNotFoundException e) {
      throw new WebServiceException("Failed to load class " + p, e);
    }
  }

  public SEILocatorBridge(SRV service,
      Class<SEI> serviceEndpointInterfaceClass, String portName,
      String lookupName) {
    this.seiClass = serviceEndpointInterfaceClass;
    initTargetBUIdRelatedLocatorTypes(serviceEndpointInterfaceClass);
    if (hasTargetBUIdRelatedLocatorTye(LOCATOR_TYPE.WS)) {
      if (isInWlsServer()) {
        locators.put(LOCATOR_TYPE.WS, new WlsSEILocator<SRV, SEI>(service,
            serviceEndpointInterfaceClass, portName, lookupName));
      } else {
        locators.put(LOCATOR_TYPE.WS, new JseSEILocator<SRV, SEI>(service,
            serviceEndpointInterfaceClass, null));
        // endpoint address is null. Use
        // ((JseSEILocator)getSEILocator(LOCATOR_TYPE.WS)).setEndpointAddress()
      }
    }
    if (hasTargetBUIdRelatedLocatorTye(LOCATOR_TYPE.EJB)) {
      locators
          .put(
              LOCATOR_TYPE.EJB,
              SmartSEILocatorFactory
                  .createLocatingPerSEIMethodInvocationSEILocator(new EjbSEILocator<SEI>(
                      serviceEndpointInterfaceClass, lookupName)));
    }
  }

  private boolean hasTargetBUIdRelatedLocatorTye(LOCATOR_TYPE type) {
    return targetBUIdRelatedLocatorTypes.containsValue(type);
  }

  private String getMappedAddress(LOCATOR_TYPE type, String address) {
    Configuration applConfig = ConfigHelper.getConfigManager()
        .getApplicationConfig();
    return applConfig.getParameter(seiClass.getName() + "." + type.toString()
        + "Address", address);
  }

  private void initTargetBUIdRelatedLocatorTypes(Map<String, String> params,
      LOCATOR_TYPE type) {
    String[] wsRelatedTargetBUIds = getStringParam(params,
        type.toString() + "RelatedTargetBUIds").split(",");
    for (String buId : wsRelatedTargetBUIds) {
      if (buId.length()>0) {
        targetBUIdRelatedLocatorTypes.put(buId, type);
      }
    }
  }

  private void initTargetBUIdRelatedLocatorTypes(
      Class<SEI> serviceEndpointInterfaceClass) {
    Configuration applConfig = ConfigHelper.getConfigManager()
        .getApplicationConfig();
    String area = serviceEndpointInterfaceClass.getName()
        + ".targetBUIdRelatedLocatorTypes";
    Map<String, String> params;
    try {
      params = applConfig.getParameters(area);
    } catch (ConfigException e) {
      throw new WebServiceException("Failed to read configuration", e);
    }
    for (Map.Entry<String, String> e : params.entrySet()) {
      targetBUIdRelatedLocatorTypes.put(e.getKey(), LOCATOR_TYPE.valueOf(e
          .getValue()));
    }
  }

  private boolean isInWlsServer() {
    return CSUtils.isInServer();
  }

  public void setSEILocator(SEILocator<SEI> locator, LOCATOR_TYPE type) {
    if (locator == null) {
      throw new NullPointerException("locator must not be null");
    }
    if (type == null) {
      throw new NullPointerException("type must not be null");
    }
    locators.put(type, locator);
  }

  public SEILocator<SEI> getSEILocator(LOCATOR_TYPE type) {
    return locators.get(type);
  }

  private LOCATOR_TYPE getLocatorType(String targetBUId) {
    if (targetBUId == null) {
      throw new NullPointerException("targetBUId is null");
    }
    LOCATOR_TYPE t = targetBUIdRelatedLocatorTypes.get(targetBUId);
    if (t == null) {
      t = targetBUIdRelatedLocatorTypes.get("default");
      if (t == null) {
        throw new WebServiceException(
            "No SEILocator configured for targetBUId " + targetBUId);
      }
    }
    return t;
  }

  /**
   * Returns a SEI which also implements the Context interface.
   */
  @SuppressWarnings("unchecked")
  public SEI getSEI() {
    final SEIContextImpl context = SEIContextFactory.createSEIContext(uri);

    InvocationHandler handler = new InvocationHandler() {
      Map<LOCATOR_TYPE, SEI> seis = new HashMap<LOCATOR_TYPE, SEI>(2);

      public Object invoke(Object proxy, Method method, Object[] args)
          throws Throwable {
        try {
          if (method.getDeclaringClass().equals(Object.class)) {
            return method.invoke(this, args);
          } else {
            try {
              SEI sei = getSEI();
              if (sei instanceof SEIContext) {
                context.copyTo((SEIContext)sei);
              }
              return method.invoke(sei, args);
            } finally {
              // open whether context should be cleared after each method.
              // context.clear();
            }
          }
        }
        catch (InvocationTargetException ie) {
          Throwable cause = ie.getCause();
          throw cause == null ? ie : cause;
        }
      }

      private SEI getSEI() {
        LOCATOR_TYPE t = getLocatorType(context.getTargetBUId());
        SEI sei = seis.get(t);
        if (sei == null) {
          sei = getSEILocator(t).getSEI();
          seis.put(t, sei);
        }
        return sei;
      }
    };

    SEI sei = (SEI)Proxy.newProxyInstance(SEILocatorBridge.class
        .getClassLoader(), new Class[] {seiClass}, handler);
    return (SEI)SEIContextDecoratorFactory.createDecorator(context, sei);
  }

}
