package com.csg.cs.ws.locator.jse;

import java.security.PrivateKey;
import java.security.cert.X509Certificate;

import org.opensaml.saml1.core.Assertion;
import org.w3c.dom.Element;

/**
 * /** A WS-Security token propagator that creates an signed SAML 1.1 assertion
 * in header of a web service request. <br>
 * The implementation is based on openSAML. The signing part is based on
 * javax.xml.crypto.dsig.<br>
 * 
 * @author A530732
 * @see OpenSamlSignedSaml11Propagator
 */
public class OpenSamlSignedSaml11Propagator2 extends
    OpenSamlUnsignedSaml11Propagator implements SigningWsSecurityPropagator {

  private PrivateKey key = null;

  private X509Certificate certificate = null;

  private JavaxDsigUtility util = new JavaxDsigUtility();

  @Override
  protected void marshallAssertion(Assertion assertion, Element parentElement)
      throws Exception {
    super.marshallAssertion(assertion, parentElement);
    util.sign(assertion.getDOM(), assertion.getSignatureReferenceID(), key,
        certificate);
  }

  @Override
  public void setSigningCredentials(PrivateKey key, X509Certificate certificate) {
    this.key = key;
    this.certificate = certificate;
  }

}
