package com.csg.cs.ws.locator.jse;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.ws.handler.soap.SOAPMessageContext;

/**
 * A WS-Security token propagator that creates an UNTP based token in header of
 * a web service request.
 * 
 * @author A530732
 * 
 */
final class UNTPPropagator implements WsSecurityPropagator {

  private static final String WSSE = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd";

  private static final String PREFIX_WSSE = "wsse";

  private static final QName QNAME_TYPE = new QName("Type");

  private static final QName QNAME_SECURITY = new QName(WSSE, "Security",
      PREFIX_WSSE);

  private PrincipalProvider principalProvider = null;

  @Override
  public void propagate(SOAPMessageContext context) throws Exception {
    /*
     * <wsse:Security
     * xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"
     * SOAP-ENV:mustUnderstand="1"> <wsse:UsernameToken> <wsse:Username>a202545</wsse:Username>
     * <wsse:Password
     * Type="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText">notUsedButRequriedInJap6</wsse:Password>
     * </wsse:UsernameToken> </wsse:Security>
     */

    SOAPMessage message = context.getMessage();
    SOAPPart part = message.getSOAPPart();
    SOAPEnvelope envelope = part.getEnvelope();
    SOAPHeader header = envelope.getHeader();
    if (header == null) {
      header = envelope.addHeader();
    }

    SOAPHeaderElement securitySoapElement = header
        .addHeaderElement(QNAME_SECURITY);
    securitySoapElement.setMustUnderstand(true);

    SOAPElement usernameToken = securitySoapElement.addChildElement(
        "UsernameToken", PREFIX_WSSE);

    SOAPElement username = usernameToken.addChildElement("Username",
        PREFIX_WSSE);
    username.addTextNode(principalProvider.getPrincipal());
    SOAPElement password = usernameToken.addChildElement("Password",
        PREFIX_WSSE);
    password
        .addAttribute(
            QNAME_TYPE,
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText");
    password.addTextNode("notUsedButRequriedInJap6");
  }

  @Override
  public void setPrincipalProvider(PrincipalProvider principalProvider) {
    this.principalProvider = principalProvider;
  }

}
