package com.csg.cs.ws.locator.jse;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

import javax.xml.namespace.QName;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPPart;
import javax.xml.ws.handler.soap.SOAPMessageContext;

/**
 * A WS-Security token propagator that creates an unsigned SAML 1.1 assertion in
 * header of a web service request. Tested against JAP 6 producer
 * 
 * @author A530732
 * @deprecated openSAML based implementation are in use now.
 * 
 */
final class UnsignedSaml11Propagator implements WsSecurityPropagator {

  private static final String WSSE = "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd";

  private static final String PREFIX_WSSE = "wsse";

  private static final String SAML = "urn:oasis:names:tc:SAML:1.0:assertion";

  private static final String PREFIX_SAML = "saml1";

  private static final QName QNAME_ATTRIBUTE_NAMESPACE = new QName(
      "AttributeNamespace");

  private static final QName QNAME_ATTRIBUTE_NAME = new QName("AttributeName");

  private static final QName QNAME_FORMAT = new QName("Format");

  private static final QName QNAME_AUTHENTICATION_INSTANT = new QName(
      "AuthenticationInstant");

  private static final QName QNAME_ATUHENTICATION_METHOD = new QName(
      "AuthenticationMethod");

  private static final QName QNAME_ISSUER_INSTANT = new QName("IssueInstant");

  private static final QName QNAME_ISSUER = new QName("Issuer");

  private static final QName QNAME_ASSERTION_ID = new QName("AssertionID");

  private static final QName QNAME_XMLNS_SAML = new QName("xmlns:saml");

  private static final QName QNAME_MINOR_VERSION = new QName("MinorVersion");

  private static final QName QNAME_MAJOR_VERSION = new QName("MajorVersion");

  private static final QName QNAME_SECURITY = new QName(WSSE, "Security",
      PREFIX_WSSE);

  private static final QName QNAME_AUTHENTICATION_STATEMENT = new QName(SAML,
      "AuthenticationStatement", PREFIX_SAML);

  private static final QName QNAME_ATTRIBUTE_VALUE = new QName(SAML,
      "AttributeValue", PREFIX_SAML);

  private static final QName QNAME_ATTRIBUTE = new QName(SAML, "Attribute",
      PREFIX_SAML);

  private static final QName QNAME_ATTRIBUTE_STATEMENT = new QName(SAML,
      "AttributeStatement", PREFIX_SAML);

  private static final QName QNAME_CONFIRMATION_METHOD = new QName(SAML,
      "ConfirmationMethod", PREFIX_SAML);

  private static final QName QNAME_SUBJECT_CONFIRMATION = new QName(SAML,
      "SubjectConfirmation", PREFIX_SAML);

  private static final QName QNAME_NAME_IDENTIFIER = new QName(SAML,
      "NameIdentifier", PREFIX_SAML);

  private static final QName QNAME_SUBJECT = new QName(SAML, "Subject",
      PREFIX_SAML);

  private static final QName QNAME_ASSERTION = new QName(SAML, "Assertion",
      PREFIX_SAML);

  private PrincipalProvider principalProvider = null;

  @Override
  public void propagate(SOAPMessageContext context) throws Exception {

    SOAPMessage message = context.getMessage();
    SOAPPart part = message.getSOAPPart();
    SOAPEnvelope envelope = part.getEnvelope();
    SOAPHeader header = envelope.getHeader();
    if (header == null) {
      header = envelope.addHeader();
    }

    String nameIdentifierValue = principalProvider.getPrincipal();

    // -START---------------create SAML1.1 security token

    Date dt1 = new Date();
    Date dt2 = new Date();
    dt2.setHours(dt2.getHours() + 1);

    SimpleDateFormat df1 = new SimpleDateFormat("yyyy-MM-dd");
    SimpleDateFormat df2 = new SimpleDateFormat("HH:mm:ss.S");

    // Security
    SOAPHeaderElement securitySoapElement = header
        .addHeaderElement(QNAME_SECURITY);
    securitySoapElement.setMustUnderstand(true);

    // Assertion
    SOAPElement assertion = securitySoapElement
        .addChildElement(QNAME_ASSERTION);
    assertion.addAttribute(QNAME_MAJOR_VERSION, "1");
    assertion.addAttribute(QNAME_MINOR_VERSION, "1");
    assertion.addAttribute(QNAME_XMLNS_SAML,
        "urn:oasis:names:tc:SAML:1.0:assertion");
    // assertion.addAttribute(factory.createName("xmlns"),
    // "urn:oasis:names:tc:SAML:1.0:assertion");
    // assertion.addAttribute(factory.createName("xmlns:samlp"),
    // "urn:oasis:names:tc:SAML:1.0:protocol");

    assertion.addAttribute(QNAME_ASSERTION_ID, "SamlSecurityToken-"
        + UUID.randomUUID().toString());
    assertion.addAttribute(QNAME_ISSUER, "http://www.credit-suisse.com");
    // assertion.addAttribute(factory.createName("IssueInstant"),
    // "2010-05-07T15:02:11.968Z");
    assertion.addAttribute(QNAME_ISSUER_INSTANT, df1.format(dt1) + "T"
        + df2.format(dt1) + "Z");

    // Condition
    // SOAPElement condition =
    // securitySoapElement.addChildElement(factory.createName("Conditions",
    // PREFIX_SAML, SAML));

    // condition.addAttribute(factory.createName("NotBefore"),
    // "2010-04-29T14:01:56.968Z");
    // condition.addAttribute(factory.createName("NotBefore"),
    // df1.format(dt1)+"T"+df2.format(dt1)+"Z");
    // condition.addAttribute(factory.createName("NotOnOrAfter"),
    // "2010-05-29T15:12:11.968Z");
    // condition.addAttribute(factory.createName("NotOnOrAfter"),
    // df1.format(dt2)+"T"+df2.format(dt2)+"Z");

    // Advice
    // SOAPElement advice =
    // securitySoapElement.addChildElement(factory.createName("Advice",
    // PREFIX_SAML, SAML));

    // AuthenticationStatement
    SOAPElement authenticationStatement = assertion
        .addChildElement(QNAME_AUTHENTICATION_STATEMENT);
    authenticationStatement.addAttribute(QNAME_ATUHENTICATION_METHOD, "x");
    // authenticationStatement.addAttribute(factory.createName("AuthenticationInstant"),
    // "2010-05-07T15:02:11.968Z");
    authenticationStatement.addAttribute(QNAME_AUTHENTICATION_INSTANT, df1
        .format(dt1)
        + "T" + df2.format(dt1) + "Z");

    // Subject
    SOAPElement subject = authenticationStatement
        .addChildElement(QNAME_SUBJECT);

    // NameIdentifier
    SOAPElement nameIdentifier = subject.addChildElement(QNAME_NAME_IDENTIFIER);
    nameIdentifier.addAttribute(QNAME_FORMAT,
        "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified");
    nameIdentifier.addTextNode(nameIdentifierValue);

    // SubjectConfirmation
    SOAPElement subjectConfirmation = subject
        .addChildElement(QNAME_SUBJECT_CONFIRMATION);

    // ConfirmationMethod
    SOAPElement confirmationMethod = subjectConfirmation
        .addChildElement(QNAME_CONFIRMATION_METHOD);
    confirmationMethod
        .addTextNode("urn:oasis:names:tc:SAML:1.0:cm:sender-vouches");

    // AttributeStatement
    SOAPElement attributeStatement = assertion
        .addChildElement(QNAME_ATTRIBUTE_STATEMENT);

    // Subject
    SOAPElement subject1 = attributeStatement.addChildElement(QNAME_SUBJECT);

    // NameIdentifier
    SOAPElement nameIdentifier1 = subject1
        .addChildElement(QNAME_NAME_IDENTIFIER);
    nameIdentifier1.addAttribute(QNAME_FORMAT,
        "urn:oasis:names:tc:SAML:1.1:nameid-format:unspecified");
    nameIdentifier1.addTextNode(nameIdentifierValue);

    // SubjectConfirmation
    SOAPElement subjectConfirmation1 = subject1
        .addChildElement(QNAME_SUBJECT_CONFIRMATION);

    // ConfirmationMethod
    SOAPElement confirmationMethod1 = subjectConfirmation1
        .addChildElement(QNAME_CONFIRMATION_METHOD);
    confirmationMethod1
        .addTextNode("urn:oasis:names:tc:SAML:1.0:cm:sender-vouches");

    // Attribute
    SOAPElement attribute = attributeStatement.addChildElement(QNAME_ATTRIBUTE);
    attribute.addAttribute(QNAME_ATTRIBUTE_NAME, "Groups");
    attribute.addAttribute(QNAME_ATTRIBUTE_NAMESPACE,
        "urn:bea:security:saml:groups");

    // AttributeValue
    SOAPElement attributeValue = attribute
        .addChildElement(QNAME_ATTRIBUTE_VALUE);
    attributeValue.addTextNode("CS_GROUP");

    // -END---------------create SAML1.1 security token
  }

  @Override
  public void setPrincipalProvider(PrincipalProvider principalProvider) {
    this.principalProvider = principalProvider;
  }

}
