package com.csg.cs.services.samples.consumer;

import com.csg.cs.core.testsupport.AbstractComponentTestCase;
import com.csg.cs.core.testsupport.TestRuntime;
import com.csg.cs.services.samples.consumer.jse.CustomerSampleRunner;

/**
 * TIPD JUnit abstract base class that includes all sample test code independent
 * of the kind of authentication.
 * <p>
 * Constraint: Concrete classes don't run against the SOA-Backplane but only against a JAP
 * based WS-Producer directly because the used PKIAuthenticationModule does an
 * authentication using a running JAP-instance.
 * <p>
 * The parameter (i.e. system property) authenticationProviderUrl is used to
 * to configure a t3(s) based URL used during authentication against a running
 * WLS instance. See also documentation of {@link CustomerSampleRunner} to get
 * a list of available parameters.
 * 
 * @author A530732
 * 
 */
public abstract class AbstractCustomerSampleServletTest extends
    AbstractComponentTestCase {

  /**
   * Default URL used for authentication.
   */
  private final static String AUTHENTICATION_PROVIDER_URL = "t3s://"
      + CustomerSampleRunner.DEFAULT_HOST_PORT;

  /**
   * CustomerSampleRunner encapsulates the sample test code.
   */
  private CustomerSampleRunner runner = new CustomerSampleRunner();

  /**
   * URL used for authentication.
   */
  private String authProviderUrl = runner.getParameter(
      "authenticationProviderUrl", AUTHENTICATION_PROVIDER_URL);

  @Override
  protected final void initRuntime(TestRuntime testRuntime) {
    testRuntime.setApplicationID("ABC1_EAR1");
    testRuntime.setUsePKI(true);
  }

  /**
   * Returns the URL used for authentication.
   */
  protected final String getAuthProviderUrl() {
    return authProviderUrl;
  }

  protected CustomerSampleRunner getCustomerSampleRunner() {
    return runner;
  }

}
