package com.csg.cs.services.samples.consumer;

import javax.net.ssl.SSLSocketFactory;

import com.csg.cs.services.samples.consumer.jse.CustomerSampleRunner;
import com.csg.cs.ws.locator.jse.ConstantPrincipalProvider;
import com.csg.cs.ws.locator.jse.CsPkiUtility;
import com.csg.cs.ws.locator.jse.PrincipalProvider;
import com.csg.cs.ws.locator.jse.WsSecurityConfigurator;

/**
 * CS_PKI based test driver to run {@link CustomerSampleRunner}.
 * <p>
 * The parameter (i.e. system property) principal is used to configure a
 * principal that should be delegated within a WS-Security token at runtime.
 * <p>
 * The parameter (i.e. system property) spidCredAlias is used to configure an
 * S-PID based SSL SocketFactory. If property is not given the MACHINE
 * certificate will be used.
 * <p>
 * See also documentation of {@link CustomerSampleRunner} to get a list of
 * available parameters.
 * 
 * @author A530732
 * 
 */
public class CustomerSampleCsPkiMain {

  public static void main(String[] args) {
    try {
      CsPkiUtility.addProvider();

      CustomerSampleRunner runner = new CustomerSampleRunner();

      String principal = runner.getParameter("principal", "S108128");

      String spidCredAlias = runner.getParameter("spidCredAlias", null);
      if (spidCredAlias != null) {
        principal = spidCredAlias;
        SSLSocketFactory socketFac = CsPkiUtility
            .createSSLSocketFactoryWithSPIDCert(spidCredAlias);
        runner.getSample1Locator().getWsSecurityConfigurator()
            .setSSLSocketFactory(socketFac);
        runner.getSample2Locator().getWsSecurityConfigurator()
            .setSSLSocketFactory(socketFac);
      }
      // else: if all is setted up fine SSLSocketFactory based on Machine-Cert
      // is
      // used. Nothing has to be implemented.
      
      String tokenType = runner.getParameter("tokenType", null);
      if (tokenType != null) {
        WsSecurityConfigurator.TokenType tp = WsSecurityConfigurator.TokenType.valueOf(tokenType);
        runner.getSample1Locator().getWsSecurityConfigurator().setTokenType(tp);
        runner.getSample2Locator().getWsSecurityConfigurator().setTokenType(tp);
      }

      PrincipalProvider principalProvider = new ConstantPrincipalProvider(
          principal);
      runner.getSample1Locator().getWsSecurityConfigurator()
          .setPrincipalProvider(principalProvider);
      runner.getSample2Locator().getWsSecurityConfigurator()
          .setPrincipalProvider(principalProvider);

      runner.run();
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

}
