package com.csg.cs.services.samples.consumer;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;

import com.csg.cs.services.samples.consumer.jse.CustomerSampleRunner;
import com.csg.cs.ws.locator.jse.ConstantPrincipalProvider;
import com.csg.cs.ws.locator.jse.PrincipalProvider;

/**
 * JKS based test driver to run {@link CustomerSampleRunner}.
 * <p>
 * The parameters (i.e. system properties) certDir, keyStore, keyStorePwd,
 * trustStore, trustStorePwd are use to locate SSL key- and trust store.
 * <p>
 * The parameter principal is used to configure the principal that has to be used
 * in a WS-Security token.
 * <p>
 * See also documentation of {@link CustomerSampleRunner} to get a list of
 * available parameters.
 * 
 * @author A530732
 */
public class CustomerSampleJksMain {

  /**
   * Reads the password from a password file. If file doesn't exist it's assumed
   * pFilename is the password itself.
   */
  private static String getPassword(String pFilename) throws IOException {
    String password = null;
    try {
      BufferedReader br;
      br = new BufferedReader(new FileReader(pFilename));
      password = br.readLine();
      br.close();
    } catch (FileNotFoundException e) {
      System.out.println("File " + pFilename + " not found, it's assumed that "
          + pFilename + " is the password");
      e.printStackTrace();
      password = pFilename;
    }
    return password;
  }

  public static void main(String[] args) {
    try {
      CustomerSampleRunner runner = new CustomerSampleRunner();

      String certDir = runner.getParameter("certDir", "certs");
      String keyStore = certDir + "/"
          + runner.getParameter("keyStore", "s108128_T.jks");
      String keyStorePwd = certDir + "/"
          + runner.getParameter("keyStorePwd", "s108128_T.jks.pwd");
      String trustStore = certDir + "/"
          + runner.getParameter("trustStore", "trust.jks");
      String trustStorePwd = certDir + "/"
          + runner.getParameter("trustStorePwd", "trust.jks.pwd");

      System.setProperty("javax.net.ssl.keyStore", keyStore);
      System.setProperty("javax.net.ssl.keyStorePassword",
          getPassword(keyStorePwd));
      System.setProperty("javax.net.ssl.trustStore", trustStore);
      System.setProperty("javax.net.ssl.trustStorePassword",
          getPassword(trustStorePwd));

      // System.setProperty("javax.net.debug", "ssl.handshake");

      String principal = runner.getParameter("principal", "S108128");
      PrincipalProvider principalProvider = new ConstantPrincipalProvider(
          principal);
      runner.getSample1Locator().getWsSecurityConfigurator()
          .setPrincipalProvider(principalProvider);
      runner.getSample2Locator().getWsSecurityConfigurator()
          .setPrincipalProvider(principalProvider);

      // To be sure to use JRE default SSLSocketFactory reset Factory in
      // WsSecurityConfigruators.
      runner.getSample1Locator().getWsSecurityConfigurator()
          .setSSLSocketFactory(null);
      runner.getSample2Locator().getWsSecurityConfigurator()
          .setSSLSocketFactory(null);

      runner.run();
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

}
