package com.csg.cs.services.samples.consumer;

import com.csg.cs.core.testsupport.security.impl.module.PIDAuthenticationModule;
import com.csg.cs.core.testsupport.security.impl.module.SSLPIDAuthenticationModule;
import com.csg.cs.corex.security.impl.module.AuthenticationModule;

/**
 * TIPD JUnit Sample based on TIPD test authenticator.
 * 
 * <p>
 * Constraint: Doesn't run against the SOA-Backplane but only against a JAP
 * based WS-Producer directly because the used PIDAuthenticationModule does an
 * authentication using a running JAP-instance.
 * <p>
 * Precondition: <br>
 * Test authenticator in producer has to be activated (see element
 * security_provider in deploy.xml)!<br>
 * System property
 * -Djava.security.properties=c:/Apps/Tip60/cs/pki/api/lib/security/java.security.providers
 * is required
 * <p>
 * The parameter (i.e. system property) authenticationPid is used to set the pid
 * that will be used in authentication.
 * <p>
 * See {@link AbstractCustomerSampleServletTest} to get more information about
 * additional parameters.
 * 
 * @author A530732
 */
public class PIDAuthenticationCustomerSampleServletTest extends
    AbstractCustomerSampleServletTest {

  /**
   * Default PID used for authentication.
   */
  private final static String AUTHENTICATION_PID = "A530732";

  /**
   * PID used for authentication.
   */
  private String authPID = getCustomerSampleRunner().getParameter(
      "authenticationPid", AUTHENTICATION_PID);

  /**
   * Runs the test code.
   */
  @org.junit.Test
  public void pidAuthenticationTest() throws Exception {
//    AuthenticationModule module = new PIDAuthenticationModule(authPID,
    AuthenticationModule module = new SSLPIDAuthenticationModule(authPID,
        getAuthProviderUrl());
    module.login();

    getCustomerSampleRunner().run();

    module.logout();
  }

}
