package com.csg.cs.services.samples.consumer;

import javax.net.ssl.SSLSocketFactory;

import com.csg.cs.corex.security.impl.module.AuthenticationModule;
import com.csg.cs.corex.security.impl.module.PKIAuthenticationModule;
import com.csg.cs.ws.locator.jse.CsPkiUtility;

/**
 * TIPD based JUnit Sample which uses an SPID-certificate. For example the
 * S108128_T located in web/WEB1/certs can be imported using
 * web/WEB1/certs/import_s108128_T.cmd.
 * 
 * <p>
 * Constraint: Doesn't run against the SOA-Backplane but only against a JAP
 * based WS-Producer directly because the used PKIAuthenticationModule does an
 * authentication using a running JAP-instance.
 * <p>
 * Precondition: <br>
 * System property
 * -Djava.security.properties=c:/Apps/Tip60/cs/pki/api/lib/security/java.security.providers
 * is required
 * <p>
 * The parameter (i.e. system property) pidCredAlias is used to set the spid
 * based CS_PKI SSLSocketFactory used in authentication an web service
 * invocation.
 * <p>
 * See {@link AbstractCustomerSampleServletTest} to get more information about
 * additional parameters.
 * 
 * 
 * @author A530732
 * 
 */
public class PKIAuthenticationCustomerSampleServletTest extends
    AbstractCustomerSampleServletTest {

  private final static String PID_CREDALIAS = "S108128";

  private String pidCredAlias = getCustomerSampleRunner().getParameter(
      "pidCredAlias", PID_CREDALIAS);

  @org.junit.Test
  public void pkiAuthenticationTest() throws Exception {
    AuthenticationModule module = new PKIAuthenticationModule(pidCredAlias,
        getAuthProviderUrl());
    module.login();

    SSLSocketFactory spidSocketFactory = CsPkiUtility
        .createSSLSocketFactoryWithSPIDCert(pidCredAlias);
    getCustomerSampleRunner().getSample1Locator().getWsSecurityConfigurator()
        .setSSLSocketFactory(spidSocketFactory);
    getCustomerSampleRunner().getSample2Locator().getWsSecurityConfigurator()
        .setSSLSocketFactory(spidSocketFactory);

    getCustomerSampleRunner().run();

    module.logout();
  }

}
