package com.csg.cs.services.samples.consumer;

import java.util.Map;

import javax.xml.ws.Binding;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.EndpointReference;

import com.csg.cs.services.inquiry.ping_1_0.PingRequest;
import com.csg.cs.services.inquiry.ping_1_0.PingResponse;
import com.csg.cs.services.tech_customersample_1.TECHCustomerSample1PortType;
import com.csg.cs.services.tech_customersample_1.createcustomer_1_0.CreateCustomer10;
import com.csg.cs.services.tech_customersample_1.createcustomer_1_0.CreateCustomer10Response;
import com.csg.cs.services.tech_customersample_1.getcustomer_1_0.GetCustomer10;
import com.csg.cs.services.tech_customersample_1.getcustomer_1_0.GetCustomer10Response;
import com.csg.cs.services.tech_customersample_1.searchcustomer_1_0.SearchCustomer10;
import com.csg.cs.services.tech_customersample_1.searchcustomer_1_0.SearchCustomer10Response;
import com.csg.cs.services.tech_customersample_2.ServiceExceptionFaultMessage;
import com.csg.cs.services.tech_customersample_2.TECHCustomerSample2PortType;
import com.csg.cs.services.tech_customersample_2.createcustomer_2_0.CreateCustomer20;
import com.csg.cs.services.tech_customersample_2.createcustomer_2_0.CreateCustomer20Response;
import com.csg.cs.services.tech_customersample_2.getcustomer_2_0.GetCustomer20;
import com.csg.cs.services.tech_customersample_2.getcustomer_2_0.GetCustomer20Response;
import com.csg.cs.services.tech_customersample_2.searchcustomer_2_0.SearchCustomer20;
import com.csg.cs.services.tech_customersample_2.searchcustomer_2_0.SearchCustomer20Response;
import com.csg.cs.ws.locator.SEIContext;

/**
 * Adapts a {@link TECHCustomerSample1PortType} to a
 * {@link TECHCustomerSample2PortType}. This enables to call a
 * TECHCustomerSample1PortType based service implementation through the
 * TECHCustomerSample2PortType.
 * 
 * @author A530732
 */
class TECHCustomerSample1PortTypeAdapter implements
    TECHCustomerSample2PortType, BindingProvider, SEIContext {

  private TECHCustomerSample1PortType sample1 = null;

  TECHCustomerSample1PortTypeAdapter(TECHCustomerSample1PortType sample1) {
    this.sample1 = sample1;
  }

  @Override
  public CreateCustomer20Response createCustomer20(
      CreateCustomer20 createcustomer20) throws ServiceExceptionFaultMessage {
    CreateCustomer10 createcustomer10 = null;
    if (createcustomer20 != null) {
      createcustomer10 = new CreateCustomer10();
      createcustomer10.setCustomer(createcustomer20.getCustomer());
    }
    try {
      CreateCustomer10Response response10 = sample1
          .createCustomer10(createcustomer10);
      CreateCustomer20Response response20 = null;
      if (response10 != null) {
        response20 = new CreateCustomer20Response();
        response20.setAttention(response10.getAttention());
        response20.setBusEx(response10.getBusEx());
        response20.setCustomerId(response10.getCustomerId());
      }
      return response20;
    } catch (com.csg.cs.services.tech_customersample_1.ServiceExceptionFaultMessage e) {
      throw adaptServiceExceptionFaultMessage(e);
    }
  }

  @Override
  public GetCustomer20Response getCustomer20(GetCustomer20 getcustomer20)
      throws ServiceExceptionFaultMessage {
    GetCustomer10 getcustomer10 = null;
    if (getcustomer20 != null) {
      getcustomer10 = new GetCustomer10();
      getcustomer10.setCustomerId(getcustomer20.getCustomerId());
    }
    try {
      GetCustomer10Response response10 = sample1.getCustomer10(getcustomer10);
      GetCustomer20Response response20 = null;
      if (response10 != null) {
        response20 = new GetCustomer20Response();
        response20.setAttention(response10.getAttention());
        response20.setBusEx(response10.getBusEx());
        response20.setCustomer(response10.getCustomer());
      }
      return response20;
    } catch (com.csg.cs.services.tech_customersample_1.ServiceExceptionFaultMessage e) {
      throw adaptServiceExceptionFaultMessage(e);
    }
  }

  @Override
  public PingResponse ping10(PingRequest pingrequest)
      throws ServiceExceptionFaultMessage {
    try {
      return sample1.ping10(pingrequest);
    } catch (com.csg.cs.services.tech_customersample_1.ServiceExceptionFaultMessage e) {
      throw adaptServiceExceptionFaultMessage(e);
    }
  }

  @Override
  public SearchCustomer20Response searchCustomer20(
      SearchCustomer20 searchcustomer20) throws ServiceExceptionFaultMessage {
    SearchCustomer10 searchcustomer10 = null;
    if (searchcustomer20 != null) {
      searchcustomer10 = new SearchCustomer10();
      searchcustomer10.setLastName(searchcustomer20.getLastName());
    }
    try {
      SearchCustomer10Response response10 = sample1
          .searchCustomer10(searchcustomer10);
      SearchCustomer20Response response20 = null;
      if (response10 != null) {
        response20 = new SearchCustomer20Response();
        response20.setAttention(response10.getAttention());
        response20.setBusEx(response10.getBusEx());
        response20.getCustomers().addAll(response10.getCustomers());
      }
      return response20;
    } catch (com.csg.cs.services.tech_customersample_1.ServiceExceptionFaultMessage e) {
      throw adaptServiceExceptionFaultMessage(e);
    }
  }

  private static com.csg.cs.services.tech_customersample_2.ServiceExceptionFaultMessage adaptServiceExceptionFaultMessage(
      com.csg.cs.services.tech_customersample_1.ServiceExceptionFaultMessage sefm1) {
    com.csg.cs.services.tech_customersample_2.ServiceExceptionFaultMessage sefm2 = new com.csg.cs.services.tech_customersample_2.ServiceExceptionFaultMessage(
        sefm1.getMessage(), sefm1.getFaultInfo(), sefm1.getCause());
    sefm2.setStackTrace(sefm1.getStackTrace());
    return sefm2;
  }

  @Override
  public Binding getBinding() {
    return ((BindingProvider)sample1).getBinding();
  }

  @Override
  public EndpointReference getEndpointReference() {
    return ((BindingProvider)sample1).getEndpointReference();
  }

  @Override
  public <T extends EndpointReference> T getEndpointReference(Class<T> clazz) {
    return ((BindingProvider)sample1).getEndpointReference(clazz);
  }

  @Override
  public Map<String, Object> getRequestContext() {
    return ((BindingProvider)sample1).getRequestContext();
  }

  @Override
  public Map<String, Object> getResponseContext() {
    return ((BindingProvider)sample1).getResponseContext();
  }

  @Override
  public void setInterface(String interf) {
    ((SEIContext)sample1).setInterface(interf);
  }

  @Override
  public void setInterfaceGroup(String interfaceGroup) {
    ((SEIContext)sample1).setInterfaceGroup(interfaceGroup);
  }

  @Override
  public void setTargetBUId(String targetBUId) {
    ((SEIContext)sample1).setTargetBUId(targetBUId);
  }

}
