package com.csg.cs.services.samples.consumer.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.csg.cs.core.base.logging.Logger;
import com.csg.cs.core.base.logging.LoggerHelper;
import com.csg.cs.services.samples.consumer.CustomerSampleRequestHandler;
import com.csg.cs.services.tech_customersample_1.TECHCustomerSample1;
import com.csg.cs.services.tech_customersample_1.TECHCustomerSample1PortType;
import com.csg.cs.services.tech_customersample_2.TECHCustomerSample2;
import com.csg.cs.services.tech_customersample_2.TECHCustomerSample2PortType;
import com.csg.cs.ws.locator.SEILocator;
import com.csg.cs.ws.locator.ThreadLocalSEILocator;
import com.csg.cs.ws.locator.j2ee.WlsSEILocator;

/**
 * Example Servlet acting as a WebService consumer to invoke the 2 services
 * TECH_CustomerSample_1 and _2. (The first one is implemented using the servlet
 * container, the second one is implemented using EJB3).
 * <p>
 * The servlet provides a simple HTML-form used to invoke the services.
 * 
 * @author A530732
 * 
 */
@SuppressWarnings("serial")
public class CustomerSampleServlet extends HttpServlet {

  private static final Logger logger = LoggerHelper
      .getLogger(CustomerSampleServlet.class);

  /**
   * A WS port is not thread-safe. One option is to have a port per thread using
   * a ThreadLocal as demonstrated here.
   */
  protected SEILocator<TECHCustomerSample1PortType> sample1Locator = null;

  /**
   * The same for second service endpoint interface.
   */
  protected SEILocator<TECHCustomerSample2PortType> sample2Locator = null;

  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    initLocators();
  }
  
  protected void initLocators() {
    sample1Locator = new ThreadLocalSEILocator<TECHCustomerSample1PortType>(
        new WlsSEILocator<TECHCustomerSample1, TECHCustomerSample1PortType>(
            TECHCustomerSample1.class, TECHCustomerSample1PortType.class,
            "TECH_CustomerSample_1_Port", "TECH_CustomerSample_1"));
    sample2Locator = new ThreadLocalSEILocator<TECHCustomerSample2PortType>(
        new WlsSEILocator<TECHCustomerSample2, TECHCustomerSample2PortType>(
            TECHCustomerSample2.class, TECHCustomerSample2PortType.class,
            "TECH_CustomerSample_2_Port", "TECH_CustomerSample_2"));
  }
  
  @Override
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException {
    this.handleRequest(request, response);
  }

  @Override
  protected void doPost(HttpServletRequest request, HttpServletResponse response)
      throws ServletException, IOException {
    this.handleRequest(request, response);
  }

  @SuppressWarnings("unchecked")
  private void handleRequest(HttpServletRequest request,
      HttpServletResponse response) throws ServletException {

    PrintWriter w = null;
    try {
      w = response.getWriter();
    } catch (IOException e) {
      throw new ServletException(
          "Failed to get PrintWriter of servlet response", e);
    }
    CustomerSampleRequestHandler.createInstance(w, request.getParameterMap(),
        sample1Locator, sample2Locator).handleRequest();
  }

}
